import { Component, OnInit, ViewChild } from '@angular/core';
import { Boilerplate } from 'src/app/models/boilerplate.model';
import { BoilerplateData } from 'src/app/models/boilerplate.model';
import { InferenceResults } from 'src/app/models/boilerplate.model';
import { BoilerplateService } from 'src/app/services/boilerplate.service';
import { PrefixService } from 'src/app/services/prefix.service';
import { ActivatedRoute, Router } from '@angular/router';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { EChartsOption } from 'echarts';
import { NotifierService } from "angular-notifier";
import { SelectionModel } from '@angular/cdk/collections';



@Component({
  selector: 'app-boilerplate-list',
  templateUrl: './boilerplate-list.component.html',
  styleUrls: ['./boilerplate-list.component.scss']
})


export class BoilerplateListComponent implements OnInit {

  chartOption: EChartsOption = {
    tooltip: {
      trigger: 'item'
    },
    legend: {
      top: '5%',
      left: 'center'
    },
    series: [
      {
        type: 'pie',
        radius: ['40%', '70%'],
        data: [
          { value: 0, name: '1' },
          { value: 0, name: '2' },
          { value: 0, name: '3' },
          { value: 0, name: '4' },
          { value: 0, name: '5' }]
      }
    ],
  };

  spinnerchart=true;
  private readonly notifier: NotifierService;
  boilerplates?: Boilerplate[];
  boilerplatesdata?: BoilerplateData[];
  currentBoilerplate?: Boilerplate;
  currentBoilerplatedata?: BoilerplateData;
  currentInfer?: InferenceResults = {
    infer_group_of_boilerplate: "",
    inference_data: "",
  }
  currentIndex = -1;
  title = '';
  displayedColumns: string[] = ['id', 'title', 'has_prefix', 'has_main', 'has_suffix'];
  newdisplayedColumns: string[] = ['select', 'title_data', 'prefix_data', 'main_data', 'suffix_data'];
  dataSource: Boilerplate[] = [{}];
  thenewdataSource: BoilerplateData[] = [{}];
  newdataSource = new MatTableDataSource(this.thenewdataSource);
  infer: InferenceResults[] = [{}];
  //when inference button is pressed
  showSpinner = false;
  exporting = false;
  selection = new SelectionModel<Boilerplate>(true, []);


  list_of_inference_metrics = ["IncompleteRequirement", "AmbiguousRequirement", "InconsistentRequirement", "NoisyRequirement", "OpaqueRequirement", "RedundantRequirement"]
  list_of_primary_inference_metrics = ["IncompleteRequirement", "AmbiguousRequirement", "InconsistentRequirement", "RedundantRequirement"]
  list_of_secondary_inference_metrics = ["NoisyRequirement", "OpaqueRequirement"]

  @ViewChild(MatPaginator) paginator = this.newdataSource.paginator;


  dataofboilerplates: any[] = [];
  aprefix: any[] = [];
  main: any;
  suffix: any;

  constructor(private boilerplateService: BoilerplateService, private prefixService: PrefixService, private route: ActivatedRoute,
    private router: Router, notifierService: NotifierService) {
    this.notifier = notifierService;
  }


  ngOnInit(): void {
    this.retrieveBoilerplates();
    this.retrieve();
    this.getInfer();
  }

  getCurrentOntology(): void {
    this.boilerplateService.getCurrentOntolgies(this.route.snapshot.params.gb)
      .subscribe(res => {
        var url = window.URL.createObjectURL(res);
        window.open(url);
      },
        error => {
          this.notifier.notify("warning", 'Fail to download ontology file');
        });

  }

  getCurrentProperties(): void {
    this.boilerplateService.getFileProperties(this.route.snapshot.params.gb)
      .subscribe(res => {
        let blob = new Blob([res], { type: 'text/plain' });
        var url = window.URL.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = url;
        a.download = 'properties.txt';
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      },
        error => {
          this.notifier.notify("warning", 'Fail to download properties file');
        });

  }


  retrieveBoilerplates(): void {
    this.boilerplateService.getAll(this.route.snapshot.params.gb)
      .subscribe(
        data => {
          this.boilerplates = data;
          this.dataSource = data;
        },
        error => {
          console.log(error);
        });

  }


  retrieve(): void {
    this.boilerplateService.getAllData(this.route.snapshot.params.gb)
      .subscribe(
        data => {
          this.boilerplatesdata = data;
          this.thenewdataSource = data;
          this.newdataSource = new MatTableDataSource(this.thenewdataSource);
          this.newdataSource.paginator = this.paginator;
          //console.log(data)
        },
        error => {
          console.log(error);
        });

  }

  refreshList(): void {
    this.retrieveBoilerplates();
    this.currentBoilerplate = undefined;
    this.currentIndex = -1;
  }

  setActiveBoilerplate(boilerplate: Boilerplate, index: number): void {
    this.currentBoilerplate = boilerplate;
    this.currentIndex = index;
  }


  SearchTitle(): void {
    if (this.title != '') {
      this.boilerplateService.findByTitle(this.route.snapshot.params.gb, this.title)
        .subscribe(
          data => {
            this.boilerplates = data;
            this.dataSource = data;
            //console.log(data);
          },
          error => {
            console.log(error);
          });

    }
    else {
      this.retrieveBoilerplates();
    }
  }
  GetRoute(): void {
    this.router.navigate(['/groupboilerplates/' + this.route.snapshot.params.gb + '/boilerplates/add-boilerplate'])
  }
  GetRoutedit(id: any): void {
    this.router.navigate(['/groupboilerplates/' + this.route.snapshot.params.gb + '/boilerplates' + id])
  }

  getInfer(): void {
    this.boilerplateService.getAllInfer(this.route.snapshot.params.gb)
      .subscribe(
        data => {
          //console.log(data);
          this.currentInfer = data[0];
          this.infer = data;
          if (data[0]) {
            const obj = JSON.parse(this.currentInfer['inference_data'].replace(/'/g, "\""))
            var final = []
            for (let i of this.list_of_inference_metrics) {
              if (obj[i] != undefined) {

                final.push({ value: obj[i].length, name: i })
              }

            }
          }
          //console.log(final);
          
          this.chartOption.series = [{
            type: 'pie',
            radius: ['40%', '70%'],
            data: final,
          }]
          this.spinnerchart = false;
        },
        error => {
          console.log(error);
        });

  }

  updateInfer(id: any): void {
    this.showSpinner = !this.showSpinner;
    this.exporting = !this.exporting
    this.boilerplateService.inferUpdate(this.route.snapshot.params.gb, id, this.currentInfer)
      .subscribe(
        data => {
          this.currentInfer = data[0];
          //this.infer = data;
          this.showSpinner = !this.showSpinner;
          this.exporting = !this.exporting;
          location.reload();
        },
        error => {
          console.log(error);
          this.showSpinner = !this.showSpinner;
          this.exporting = !this.exporting
          this.notifier.notify("warning", (JSON.stringify(error["error"]["detail"])));
        });
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.newdataSource.filter = filterValue.trim().toLowerCase();
  }

  get_metrics(i: any): any {
    if (this.currentInfer?.inference_data) {
      const obj = JSON.parse(this.currentInfer['inference_data'].replace(/'/g, "\""))
      var final = []
      if (obj[i]) {
        for (let j in obj[i]){ 
          //console.log(obj[i][j])
          if(this.list_of_primary_inference_metrics.includes(obj[i][j])){
            final.push(obj[i][j])
          }
        }
        //return obj[i]
      }
      //final.push({ value: obj[i].length, name: i })
      return final;
    }
    return 0
  }

  get_secondary_metrics(i: any): any {
    if (this.currentInfer?.inference_data) {
      const obj = JSON.parse(this.currentInfer['inference_data'].replace(/'/g, "\""))
      var final = []
      if (obj[i]) {
        for (let j in obj[i]){ 
          //console.log(obj[i][j])
          if(this.list_of_secondary_inference_metrics.includes(obj[i][j])){
            final.push(obj[i][j])
          }
        }
        //return obj[i]
      }
      //final.push({ value: obj[i].length, name: i })
      return final;
    }
    return 0
  }


  getnumreq(): any {
    return this.boilerplatesdata?.length
  }

  getincreq(): any {
    if (this.get_metrics('IncompleteRequirement').length > 0) {
      return this.get_metrics('IncompleteRequirement').length;
    }
    else return 0;
  }

  /** Whether the number of selected elements matches the total number of rows. */
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.newdataSource.data.length;
    //console.log(this.selection.selected[0].id);
    return numSelected === numRows;
  }

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  masterToggle() {
    this.isAllSelected() ?
      this.selection.clear() :
      this.newdataSource.data.forEach(row => this.selection.select(row));
  }
  deleteBoilerplateList(): void {
    if (confirm("Αre you sure you want to delete this list of Boilerplates?")) {
      let num = 0;
      for (let currentboilerplate of this.selection.selected) {
        num++;
        this.boilerplateService.delete(this.route.snapshot.params.gb, currentboilerplate.id)
          .subscribe(
            response => {
              if (num == this.selection.selected.length) { location.reload(); }
            },
            error => {
              this.notifier.notify("warning", (JSON.stringify(error)));
            });
      }
    }
    // location.reload();
  }

}
